﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Media;

namespace SerialIO
{

    public partial class SerialIOform : Form
    {
        string[]            PortNames = {};
        int                 BaudRateDefault = 9600;
        Parity              ParityDefault = Parity.None;
        int                 DataBitsDefault = 8;
        StopBits            StopBitsDefault = StopBits.One;
        Handshake           HandshakeDefault = Handshake.None;
        bool                RTSdefault = false;
        bool                DTRdefault = false;

        const string        MessagePortError = "Error opening Port";

        // Create a new SerialPort object with default settings.
        SerialPort IOport = new SerialPort();

        // Create timer to do stuff when started
        Timer DoItNow = new Timer();

        public SerialIOform()
        {
            InitializeComponent();
            // Set port defaults and check if ports exist
            SerialIOPortSetup();
            // Setup timer
            DoItNow.Interval = 500; // Two times a second
            DoItNow.Tick += new EventHandler(DoItNow_Tick);
        }

        // Setup Defaults & Dropdown
        private void SerialIOPortSetup()
        {
            PortNames = SerialPort.GetPortNames();
            if (PortNames.Length != 0)
            {
                // Set Defaults
                IOport.PortName = PortNames[0];
                IOport.BaudRate = BaudRateDefault;
                IOport.Parity = ParityDefault;
                IOport.DataBits = DataBitsDefault;
                IOport.StopBits = StopBitsDefault;
                IOport.Handshake = HandshakeDefault;
                IOport.RtsEnable = RTSdefault;
                IOport.DtrEnable = DTRdefault;

                // Populate Dropdown
                SerialPortComboBox.Items.Clear();
                SerialPortComboBox.Items.AddRange(PortNames);
                SerialPortComboBox.SelectedIndex = 0;
                MessagesLabel.Text = "";
                StartButton.Text = "Start";
            }
            else
            {
                SerialPortComboBox.Items.Clear();
                MessagesLabel.Text = "No ports found! Connect & press Refresh";
                StartButton.Text = "Refresh";
            }
        }

        private void SerialIOform_Load(object sender, EventArgs e)
        {
                
        }

        private void SerialPortComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // close old port
            if (IOport.IsOpen)
                IOport.Close();

            // set to selected port
            IOport.PortName = SerialPortComboBox.SelectedItem.ToString();

            // Check port valid
            try
            {
                IOport.Open();
                MessagesLabel.Text = "";  // clear old error msg
            }
            catch
            {
                MessagesLabel.Text = MessagePortError;
            }
            // Close it so it isn't open when doing nothing, 
            // Saves problem when port unplugged
            if (IOport.IsOpen)
                IOport.Close();
        }

        private void StartButton_Click(object sender, EventArgs e)
        {
            if (StartButton.Text == "Refresh")
            {
                SerialIOPortSetup();
            }
            else if (StartButton.Text == "Stop")
            {
                // Stop timer
                DoItNow.Stop();
                IOport.Close();
                StartButton.Text = "Start";
            }
            else // must be "Start"
            {
                if (IOport.IsOpen )   // safety belt, port should be closed
                    IOport.Close();
                // Check port valid, need to do it again as device could be unplugged
                try {
                    IOport.Open();
                }
                catch {
                    MessagesLabel.Text = MessagePortError;
                    return;
                }
                StartButton.Text = "Stop";
                // Port opened, Start timer to do stuff
                DoItNow.Start();
            }
        }

        void DoItNow_Tick(object sender, EventArgs e)
        {
            // Stop timer while processing event
            DoItNow.Stop();
            // Set input checkboxes to IOport settings
            DCDcheckBox.Checked = IOport.CDHolding;
            DSRcheckBox.Checked = IOport.DsrHolding;

            // Beep if set
            if (DCDbeepCheckBox.Checked && DCDcheckBox.Checked)
                SystemSounds.Asterisk.Play();
 
            if (DSRbeepCheckBox.Checked && DSRcheckBox.Checked)
                SystemSounds.Beep.Play();

            // Toggle if blink checked
            if (DTRblinkCheckBox.Checked)
                DTRcheckBox.Checked = !DTRcheckBox.Checked;
            if (RTSblinkCheckBox.Checked)
                RTScheckBox.Checked = !RTScheckBox.Checked;
        
            // Set IOport to output settings
            IOport.DtrEnable = DTRcheckBox.Checked;
            IOport.RtsEnable = RTScheckBox.Checked;

            // Restart timer
            DoItNow.Start();
        }

        private void CancelButton_Click(object sender, EventArgs e)
        {
            if (IOport.IsOpen ) 
                IOport.Close();
            // get rid of IOport??

            // exit
            SerialIOform.ActiveForm.Close();
        }

        private void DCDbeepCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            // don't need to do stuff, we test checked state elsewhere
        }

        private void DSRbeepCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            // don't need to do stuff, we test checked state elsewhere
        }

        private void DTRblinkCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            // don't need to do stuff, we test checked state elsewhere
        }
        private void DTRBlinkCheckBox_CheckedChanged(object sender, EventArgs e) 
        // Visual Studio got its nickers in a knot when I renamed DTRBlink checkbox to DTRblink
        // and now asks for a DTRBlink version. ???
        {
            // don't need to do stuff, we test checked state elsewhere
        }

        private void RTSblinkCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            // don't need to do stuff, we test checked state elsewhere
        }

        private void DTRcheckBox_CheckedChanged(object sender, EventArgs e)
        {
            // don't need to do stuff, we set state elsewhere
        }

        private void RTScheckBox_CheckedChanged(object sender, EventArgs e)
        {
            // don't need to do stuff, we set state elsewhere
        }

        //
        //
    } // SerialIOForm
} // SerialIO namespace
