﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Media;

namespace SerialWrite
{

    public partial class SerialIOform : Form
    {
        string[]            PortNames = {};
        int                 BaudRateDefault = 9600;
        Parity              ParityDefault = Parity.None;
        int                 DataBitsDefault = 8;
        StopBits            StopBitsDefault = StopBits.One;
        Handshake           HandshakeDefault = Handshake.None;
        bool                RTSdefault = false;
        bool                DTRdefault = false;
        int                 TimeoutDefault = 500;   // Windows default is no timeout, it will block
                                                    // program if nothing connected to port

        const string        MessagePortError = "Error opening Port";
        const string        MessageWriteError = "Error writing to port";

        // Secret Messages

        // const string     Msg1 = "ABC123`n"; // short test msg
        const string        Msg1 = "Watson, come here! I want to see you!";
        const string        Msg2 = "Wikileaks caused the radiation leaks in Japan";
        const string        Msg3 = "All your Wixels are belong to us";

        // Create a new SerialPort object with default settings.
        SerialPort IOport = new SerialPort();

        public SerialIOform()
        {
            InitializeComponent();
            // Set port defaults and check if ports exist
            SerialIOPortSetup();
        }

        // Setup Defaults & Dropdown
        private void SerialIOPortSetup()
        {
            PortNames = SerialPort.GetPortNames();
            if (PortNames.Length != 0)
            {
                // Set Defaults
                IOport.PortName = PortNames[0];
                IOport.BaudRate = BaudRateDefault;
                IOport.Parity = ParityDefault;
                IOport.DataBits = DataBitsDefault;
                IOport.StopBits = StopBitsDefault;
                IOport.Handshake = HandshakeDefault;
                IOport.RtsEnable = RTSdefault;
                IOport.DtrEnable = DTRdefault;
                IOport.WriteTimeout = TimeoutDefault;
                IOport.ReadTimeout = TimeoutDefault;

                // Populate Dropdown
                SerialPortComboBox.Items.Clear();
                SerialPortComboBox.Items.AddRange(PortNames);
                SerialPortComboBox.SelectedIndex = 0;
                MessagesLabel.Text = "";
                SerialPortComboBox.Enabled = true;
            }
            else
            {
                SerialPortComboBox.Items.Clear();
                SerialPortComboBox.Enabled = false;
                MessagesLabel.Text = "No ports found! Connect device && Double Click this Message";
            }
        } // SerialPortPortSetup

        private void SerialIOform_Load(object sender, EventArgs e)
        {
                
        }

        private void SerialPortComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // close old port
            if (IOport.IsOpen)  // safetybelt - should be closed
                IOport.Close();

            // set to selected port
            IOport.PortName = SerialPortComboBox.SelectedItem.ToString();
            if (!OpenPort())
            {
                MessagesLabel.Text = MessagePortError;
                button1.Enabled = false;
                button2.Enabled = false;
                button3.Enabled = false;
            }
            else
            {
                // Close it so it isn't open when doing nothing, 
                // Saves problem when port unplugged
                if (IOport.IsOpen)
                    IOport.Close();
                MessagesLabel.Text = "";
                button1.Enabled = true;
                button2.Enabled = true;
                button3.Enabled = true;
            }
        } // SerialPortComboBox_SelectedIndexChanged


        private void CancelButton_Click(object sender, EventArgs e)
        {
            if (IOport.IsOpen ) 
                IOport.Close();
            // get rid of IOport??

            // exit
            SerialIOform.ActiveForm.Close();
        } // Cancel_Click

        private void button1_Click(object sender, EventArgs e)
        {
            if (!OpenPort())
            {
                MessagesLabel.Text = MessagePortError;
                return;
            }
            if (!WritePort(Msg1))
                MessagesLabel.Text = "Write error";
            else
                MessagesLabel.Text = Msg1;
            IOport.Close();
        } // b1_Click

        private void button2_Click(object sender, EventArgs e)
        {
            if (!OpenPort())
            {
                MessagesLabel.Text = MessagePortError;
                return;
            }
            if (!WritePort(Msg2))
                MessagesLabel.Text = "Write error";
            else
                MessagesLabel.Text = Msg2;
            IOport.Close();
        } // b2_Click

        private void button3_Click(object sender, EventArgs e)
        {
            if (!OpenPort())
            {
                MessagesLabel.Text = MessagePortError;
                return;
            }
            if (!WritePort(Msg3))
                MessagesLabel.Text = "Write error";
            else
                MessagesLabel.Text = Msg3;
            IOport.Close();
        } // b3_Click

        private void MessagesLabel_Click(object sender, EventArgs e)
        {
            SerialIOPortSetup();
        }

        bool WritePort(String t) {
            if (IOport.IsOpen)
                try
                {
                    IOport.WriteLine(t);
                    return true;
                }
                catch
                {
                    return false;
                }
            else
                    return false;

        } // WritePort

        bool OpenPort()
        {
            if (IOport.IsOpen)   // safety belt, port should be closed
                IOport.Close();
            // Check port valid, need to do it again as device could be unplugged
            try
            {
                IOport.Open();
            }
            catch
            {
                return false;
            }
            return true;
        } // OpenPort
        //
    } // SerialIOForm
} // SerialWrite namespace
